@@ -328,7 +328,20 @@ class Scheduler(PollScheduler):
 	def _terminate_tasks(self):
 		self._status_display.quiet = True
 		for task in list(self._running_tasks.values()):
-			task.cancel()
+			if task.isAlive():
+				# This task should keep the main loop running until
+				# it has had an opportunity to clean up after itself.
+				# Rely on its exit hook to remove it from
+				# self._running_tasks when it has finished cleaning up.
+				task.cancel()
+			else:
+				# This task has been waiting to be started in one of
+				# self._task_queues which are all cleared below. It
+				# will never be started, so purged it from
+				# self._running_tasks so that it won't keep the main
+				# loop running.
+				del self._running_tasks[id(task)]
+
 		for q in self._task_queues.values():
 			q.clear()
 
@@ -603,9 +616,6 @@ class Scheduler(PollScheduler):
 		tasks are started.
 		"""
 
-		if '--fetchonly' in self.myopts:
-			return os.EX_OK
-
 		digest = '--digest' in self.myopts
 		if not digest:
 			for pkgsettings in self.pkgsettings.values():
@@ -646,38 +656,6 @@ class Scheduler(PollScheduler):
 
 		return os.EX_OK
 
-	def _env_sanity_check(self):
-		"""
-		Verify a sane environment before trying to build anything from source.
-		"""
-		have_src_pkg = False
-		for x in self._mergelist:
-			if isinstance(x, Package) and not x.built:
-				have_src_pkg = True
-				break
-
-		if not have_src_pkg:
-			return os.EX_OK
-
-		for settings in self.pkgsettings.values():
-			for var in ("ARCH", ):
-				value = settings.get(var)
-				if value and value.strip():
-					continue
-				msg = _("%(var)s is not set... "
-					"Are you missing the '%(configroot)s%(profile_path)s' symlink? "
-					"Is the symlink correct? "
-					"Is your portage tree complete?") % \
-					{"var": var, "configroot": settings["PORTAGE_CONFIGROOT"],
-					"profile_path": portage.const.PROFILE_PATH}
-
-				out = portage.output.EOutput()
-				for line in textwrap.wrap(msg, 70):
-					out.eerror(line)
-				return FAILURE
-
-		return os.EX_OK
-
 	def _check_manifests(self):
 		# Verify all the manifests now so that the user is notified of failure
 		# as soon as possible.
@@ -992,10 +970,6 @@ class Scheduler(PollScheduler):
 		if rval != os.EX_OK:
 			return rval
 
-		rval = self._env_sanity_check()
-		if rval != os.EX_OK:
-			return rval
-
 		# TODO: Immediately recalculate deps here if --keep-going
 		#       is enabled and corrupt manifests are detected.
 		rval = self._check_manifests()
@@ -1042,6 +1016,7 @@ class Scheduler(PollScheduler):
 				else:
 					signal.signal(signal.SIGCONT, signal.SIG_DFL)
 
+			self._termination_check()
 			if received_signal:
 				sys.exit(received_signal[0])
 
@@ -1078,6 +1053,10 @@ class Scheduler(PollScheduler):
 				if isinstance(x, Package) and x.operation == "merge"])
 			self._status_display.maxval = self._pkg_count.maxval
 
+		# Cleanup any callbacks that have been registered with the global
+		# event loop by calls to the terminate method.
+		self._cleanup()
+
 		self._logger.log(" *** Finished. Cleaning up...")
 
 		if failed_pkgs:
@@ -1380,7 +1359,6 @@ class Scheduler(PollScheduler):
 		blocker_db.discardBlocker(pkg)
 
 	def _main_loop(self):
-		term_check_id = self._event_loop.idle_add(self._termination_check)
 		loadavg_check_id = None
 		if self._max_load is not None and \
 			self._loadavg_latency is not None and \
@@ -1407,7 +1385,6 @@ class Scheduler(PollScheduler):
 			while self._is_work_scheduled():
 				self._event_loop.iteration()
 		finally:
-			self._event_loop.source_remove(term_check_id)
 			if loadavg_check_id is not None:
 				self._event_loop.source_remove(loadavg_check_id)
 
